//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CUBICSPLINERECON_H
#define CUBICSPLINERECON_H

#include <Eigen/Core>
#include <QPointF>
#include "MonotoneInterpolator.h"
#include "Parameters.h"
class Sequence;
class QProgressDialog;

class CubicSplineRecon
{
public:
    CubicSplineRecon(const Eigen::MatrixXd &targetPoints,
                     const Parameters::Tolerance &tol,
                     Sequence &sequence,
                     double paramPower = 0.5,
                     QProgressDialog *progress = NULL);
    CubicSplineRecon(Sequence &sequence,
                     QPointF position,
                     int numControlPoints);
    
    void compute(const Eigen::MatrixXd &targetPoints, int numControlPoints);
    void directEvaluate(double t, Eigen::VectorXd &result, int nDerivs = 0, bool onIdentity = false) const;
    QPointF directEvaluate(double t, int nDerivs = 0) const;
    void evaluate(double t, Eigen::VectorXd &result, int nDerivs = 0) const;
    QPointF evaluate(double t, int nDerivs = 0) const;
    int numPoints() const { return controlPoints.rows(); }
    int maxOfDomain() const { return numSamples - 1; }
    const std::vector<double> &getKnots() const { return knots; }
    const Eigen::MatrixXd &getControlPoints() const { return controlPoints; }
    void setControlPoint(int pointNumber, QPointF newPoint);
    void uniformResample(int nFrames);
    void addFrozenFrames(int afterFrame, int nFrames, double newFrameVal);
    void duplicateFrames(int fromFrame, int toFrame, int repetitions);
    MonotoneInterpolator &getMonotoneInterpolant() { return paramInterp; }
    const double getResampleFactor() const { return resampleFactor; }

private:
    void computeParams(double alpha, const Eigen::MatrixXd &targetPoints);

    Sequence &sequence;
    double paramPower;
    double resampleFactor;
    std::vector<double> params;
    std::vector<double> knots;
    Eigen::MatrixXd controlPoints;
    MonotoneInterpolator paramInterp;
    int numSamples;
};

#endif // CUBICSPLINERECON_H
