//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef DEFGRADREP_H
#define DEFGRADREP_H

#include "Rep.h"
#include <Eigen/Core>
#include "openmesh.h"

class DefGradRep : public Rep {
    friend class DefGradRepConverter;

public:
    DefGradRep() {}
    DefGradRep(const DefGradRepConverter &converter, const Mesh &mesh);

    Eigen::VectorXd getVector() const;
    void setFromVector(const Eigen::VectorXd &vec);
    Description getDescription() const;

    DefGradRep operator+(const DefGradRep &other) const;
    DefGradRep operator-(const DefGradRep &other) const;
    DefGradRep operator*(double mult) const;
    DefGradRep &operator+=(const DefGradRep &other);
    DefGradRep &operator-=(const DefGradRep &other);
    DefGradRep &operator*=(double mult);

private:
    Eigen::MatrixXd rots;
    Eigen::MatrixXd scaleShears;
    Eigen::Vector3d barycenter;
};

inline DefGradRep operator*(double mult, const DefGradRep &other)
{
    return other * mult;
}

#endif // DEFGRADREP_H
