//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef GRAPHICSVIEW_H
#define GRAPHICSVIEW_H

#include <QGraphicsView>

class GraphicsView : public QGraphicsView
{
    Q_OBJECT

public:
    explicit GraphicsView(QWidget *parent = 0);
    void resizeToFit();
    void setCurrentPoint(QPointF pos);
    QPointF getCurrentPoint() const;
    void setScene(QGraphicsScene *newScene);
    bool mouseEventsEnabled() const { return mouseEventsFlag; }
    void setMouseEventsEnabled(bool enabled) { mouseEventsFlag = enabled; }
    void setCurrentPointVisible(bool visible);
    void setAspectRatioMode(Qt::AspectRatioMode aspect) { aspectRatioMode = aspect; }

signals:
    void newPointPosition(double x, double y);

protected:
    void resizeEvent(QResizeEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);

private:
    void detachFromScene();

    QGraphicsEllipseItem *currentPoint;
    Qt::AspectRatioMode   aspectRatioMode;
    bool mouseEventsFlag;
    bool currentPointVisibleFlag;
};

#endif // GRAPHICSVIEW_H