//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <Eigen/Core>
#include <QLabel>
#include <QMainWindow>
#include "openmesh.h"
#include "Parameters.h"
class OpenMeshDrawer;
class VertexRepConverter;
class DefGradRepConverter;
class LinRotInvRepConverter;
class Sequence;

namespace Ui {
    class MainWindow;
    class ParametersDialog;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    ~MainWindow();
    static MainWindow *getInstance() {
        if(!instance) instance = new MainWindow();
        return instance;
    }

public slots:
    void evaluateSpline(double t);
    void evaluateThinPlate(double x, double y);
    void setMeshFromVector(Eigen::VectorXd &vec);
    void setFitSpline(bool fit);
    void splineCurveChanged();

protected:
    explicit MainWindow(QWidget *parent = 0);

private slots:
    void on_actionLoadSequence_triggered();

    void on_actionParameters_triggered();
    void on_actionVertex_triggered();
    void on_actionDefGrad_triggered();
    void on_actionLinRotInv_triggered();

    void on_actionReconstructFromShapes_triggered();
    void on_actionSwitchSubspace_triggered();
    void on_actionViewPoseSpaceConnections_toggled(bool checked);
    void on_actionViewPoseSpacePositions_toggled(bool checked);
    void on_actionFitSpline_toggled(bool checked);
    void on_actionCreateUniformSpline_triggered();
    void on_actionUniformResample_triggered();
    void on_actionNonuniformResample_triggered();
    void on_actionAddFrozenFrames_triggered();
    void on_actionDuplicateFrames_triggered();
    
    void on_snapToCurveCheckBox_toggled(bool checked);
    void on_showSourceCheckBox_toggled(bool checked);
    void on_playStopButton_clicked();
    void on_timeSlider_valueChanged(double value);
    void on_tabs_currentChanged(int index);
    void on_keyframeSpinBox_valueChanged(int value);
    void on_keyframeScaleSpinBox_valueChanged(double value);
    void on_addPoseButton_clicked();
    void on_unpinSelectedButton_clicked();
    void on_unpinAllButton_clicked();

    void advanceFrame();
    void startPlaying();
    void stopPlaying();

private:
    static MainWindow *instance;
    enum RepOption { Vertex, DefGrad, LinRotInv };

    void initConvertersFromTarget();
    void connectSequences();
    void setCheckedRep() const;
    void loadSequence(QString fileNameRoot,
                      int fromFrame, int toFrame,
                      int numberWidth, QString extension,
                      bool createSequences = true);
    void compressCurrentRep();
    void setDialogFromParameters(Ui::ParametersDialog &dialog) const;
    void setParametersFromDialog(const Ui::ParametersDialog &dialog);

    Ui::MainWindow *ui;
    OpenMeshDrawer *drawer;
    Mesh            target;
    
    QTimer         *animTimer;
    bool            playing;
    int             fromFrame, toFrame;

    std::map<RepOption, Sequence *> sequences;
    RepOption currentRep;
    
    VertexRepConverter    *vConverter;
    DefGradRepConverter   *dConverter;
    LinRotInvRepConverter *lConverter;
    Parameters             parameters;

    QLabel sourceStatus;
    QLabel pcaStatus;
    QLabel thinPlateStatus;
    QLabel splineStatus;
    bool   statusLabelsAdded;
};

#endif // MAINWINDOW_H
