//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef REP_H
#define REP_H

#include <Eigen/Core>
#include <vector>

class Rep {
public:
    struct Description {
        struct Part {
            Part(const char *desc, int rows) : desc(desc), rows(rows) {}
            int rows;
            const char *desc;
        };
        std::vector<Part> parts;

        int rows() const {
            int rows = 0;
            for (std::vector<Part>::const_iterator p = parts.begin();
                 p != parts.end(); ++p)
            {
                rows += p->rows;
            }
            return rows;
        }
    };

    virtual ~Rep() {}
    virtual Eigen::VectorXd getVector() const = 0;
    virtual void setFromVector(const Eigen::VectorXd &vec) = 0;
    virtual Description getDescription() const = 0;
};

#endif // REP_H
