//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef SUBSPACERECON_H
#define SUBSPACERECON_H

#include <Eigen/Core>
#include "Parameters.h"
class Sequence;
class QProgressDialog;

class SubspaceRecon {
public:
    SubspaceRecon(const Sequence &sequence);

    void initSubspace(int nComps);
    void setOffset(const Eigen::VectorXd &newOffset);
    void setSubspaceVector(int comp, const Eigen::VectorXd &vector);
    void computeLSCoeffs(const Eigen::MatrixXd &normalizedFeatures);
    void resetSubspace();

    void computePCA(const Eigen::MatrixXd &normalizedFeatures,
                    const Parameters::Tolerance &tol,
                    QProgressDialog *progress = NULL);

    void reconstructFrame(int col, Eigen::VectorXd &result, int nComps = -1) const;
    void reconstructAllFrames(Eigen::MatrixXd &result, int nComps = -1) const;
    void reconstructFrom(const Eigen::VectorXd &cs, Eigen::VectorXd &result) const;
    void reconstructFrom(const Eigen::MatrixXd &cs, Eigen::MatrixXd &result) const;
    int numComps() const { return useComps; }
    double getCoeff(int frame, int comp) const { return coeffs(comp, frame); }
    void getCoeffSubMatrix(Eigen::MatrixXd &result) const;

private:
    const Sequence &sequence;
    Eigen::VectorXd offset;
    Eigen::MatrixXd subspace;
    Eigen::MatrixXd coeffs;
    int useComps;
};

#endif // SUBSPACERECON_H
