//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef THINPLATERECON_H
#define THINPLATERECON_H

#include <Eigen/Core>
#include <vector>
#include "Parameters.h"
class Sequence;
class QProgressDialog;

class ThinPlateRecon
{
public:
    ThinPlateRecon(const Parameters::Tolerance &tol,
                   const Sequence &sequence,
                   bool haveGroundTruth,
                   double smoothnessParameter = 1.0,
                   QProgressDialog *progress = NULL);
    void evaluate(double x, double y, Eigen::VectorXd &dest) const;
    int numIncludedSites() const;
    bool isFrameIncluded(int frame) const { return included.at(frame); }

private:
    void calculate();
    
    const Parameters::Tolerance &tol;
    const Sequence &sequence;
    double p;

    Eigen::MatrixXd X;
    Eigen::MatrixXd q2;
    Eigen::MatrixXd pcaCoeffs;
    Eigen::MatrixXd coeffs;
    Eigen::MatrixXd linearPart;
    std::vector<bool> included;
};

template<typename Iter>
class IndexCompare {
private:
   Iter _iter;
public:
   IndexCompare(Iter iter):_iter(iter)
   {}
   
   bool operator()(size_t i, size_t j) const
   {
      return *(_iter+i) < *(_iter+j);
   }
};

template<typename Iter>
IndexCompare<Iter> index_compare(Iter iter)
{
   return IndexCompare<Iter>(iter);
}

template<typename Iter>
class IndexCompareGreater {
private:
   Iter _iter;
public:
   IndexCompareGreater(Iter iter):_iter(iter)
   {}
   
   bool operator()(size_t i, size_t j) const
   {
      return *(_iter+i) > *(_iter+j);
   }
};

template<typename Iter>
IndexCompareGreater<Iter> index_compare_greater(Iter iter)
{
   return IndexCompareGreater<Iter>(iter);
}

#endif // THINPLATERECON_H
