//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef VERTEXREP_H
#define VERTEXREP_H

#include "Rep.h"
#include <Eigen/Core>
#include "openmesh.h"

class VertexRep : public Rep {
    friend class VertexRepConverter;

public:
    VertexRep() {}
    VertexRep(const VertexRepConverter &converter, const Mesh &mesh);

    Eigen::VectorXd getVector() const;
    void setFromVector(const Eigen::VectorXd &vec);
    Description getDescription() const;

    VertexRep operator+(const VertexRep &other) const;
    VertexRep operator-(const VertexRep &other) const;
    VertexRep operator*(double mult) const;
    VertexRep &operator+=(const VertexRep &other);
    VertexRep &operator-=(const VertexRep &other);
    VertexRep &operator*=(double mult);

private:
    Eigen::VectorXd coords;
};

inline VertexRep operator*(double mult, const VertexRep &other)
{
    return other * mult;
}

#endif // VERTEXREP_H
