//
//    Copyright (c) 2011 Thomas J. Cashman and Kai Hormann
//                       Tom.Cashman@cantab.net / Kai.Hormann@usi.ch
//
//    This file is part of DefMeshRep.
//
//    DefMeshRep is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    DefMeshRep is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with DefMeshRep.  If not, see <http://www.gnu.org/licenses/>.
//

#include "VertexRep.h"
#include "VertexRepConverter.h"

VertexRep::VertexRep(const VertexRepConverter &converter, const Mesh &mesh)
{
    converter.setRepFromMesh(*this, mesh);
}

Eigen::VectorXd VertexRep::getVector() const
{
    return coords;
}

void VertexRep::setFromVector(const Eigen::VectorXd &vec)
{
    coords = vec;
}

Rep::Description VertexRep::getDescription() const
{
    Description d;
    d.parts.push_back(Description::Part("Vertices", coords.size()));
    return d;
}

VertexRep VertexRep::operator+(const VertexRep &other) const
{
    VertexRep ret(*this);
    ret += other;
    return ret;
}

VertexRep VertexRep::operator-(const VertexRep &other) const
{
    VertexRep ret(*this);
    ret -= other;
    return ret;
}

VertexRep VertexRep::operator*(double mult) const
{
    VertexRep ret(*this);
    ret *= mult;
    return ret;
}

VertexRep &VertexRep::operator+=(const VertexRep &other)
{
    coords += other.coords;
    return *this;
}

VertexRep &VertexRep::operator-=(const VertexRep &other)
{
    coords -= other.coords;
    return *this;
}

VertexRep &VertexRep::operator*=(double mult)
{
    coords *= mult;
    return *this;
}
